{ Program to test sped of StrLoadFile functions }

program project1;
{$mode objfpc}
{$H+}

// Renamed cpu to cpu-timer, because the existence of a cpu-unit in the RTL
uses
  cpu_timer, sysutils, classes;


var
  start, stop : int64;
  s,t         : string;
  i           : integer;

const
 {$ifdef WIN32}
  CRLF: string = #13#10; // windows uses CRLF (carriage return and a line feed)
 {$endif}

 {$ifdef UNIX}
  CRLF: string = #10;    // linux uses LF. (line feed)
 {$endif}

 {$ifdef MACOS}
  CRLF: string = #13;    // mac uses CR (carriage return)
 {$endif}



{ get the size of any file, doesn't matter whether it's a text or binary file. }
function GetFileSize(const FileName: string): longint;
var
  FileInfo:TSearchRec;
begin
  result:= 0;
  FindFirst(FileName, faAnyFile, FileInfo);
  result:= FileInfo.Size ;
end;


function StrLoadFile_test1(const FileName: string): string;
var
  F: text;
  c: char;
  str1,Line: string;
begin
  result:= ''; //safety
  str1:= '';
  if FileExists(FileName) = false then
  begin
    result:= ''; //returns nothing if file not found
    exit;
  end;
  Assign(F, FileName);   //open file
  Reset(F);              //
  while not Eof(F) do
  begin
    Read(F, C);
    result:= result + C;
  end;
  close(F); //close file
end;


function StrLoadFile_test2(const FileName: string): string;
var
  F: file;
  InputFileSize: Integer;
  Buffer: Pointer;
begin
  if FileExists(FileName) = false then
  begin
    result:= ''; //returns nothing if file not found
    exit;
  end;
  FileMode:= fmOpenRead;
  AssignFile(F, FileName);
  Reset(F, 1);
  InputFileSize:= FileSize(F);
  SetLength(Result, InputFileSize);
  Buffer:= PChar(Result);
  BlockRead(F, Buffer^, InputFileSize);
  CloseFile(F);
end;

function StrLoadFile_test3(const FileName: string): string;
var
  F: file;
  InputFileSize: Integer;
  Buffer: Pointer;
begin
  if FileExists(FileName) = false then
  begin
    result:= ''; //returns nothing if file not found
    exit;
  end;
  FileMode:= fmOpenRead;
  AssignFile(F, FileName);
  InputFileSize:= GetFileSize(Filename);
  Reset(F, InputFileSize);
  SetLength(Result, InputFileSize);
  Buffer:= PChar(Result);
  BlockRead(F, Buffer^, 1);
  CloseFile(F);
end;


const
  count = 10;
  demofile = 'demotext.big.txt';
var
  TestString: string;
  TestStrList: TStringList;
begin
  writeln('Welcome to the CPU war for StrLoadFile.. Please wait...');
  writeln;
  
  start:= Clock;
  for i:= 0 to count do
    TestString:= StrLoadFile_Test1(demofile);
  stop := Clock;
  writeln('test 1 execution time: ',stop-start);

  start:= Clock;
  for i:= 0 to count do
    TestString:= StrLoadFile_Test2(demofile);
  stop := Clock;
  writeln('test 2 execution time: ',stop-start);

  start := Clock;
  for i := 0 to count do
    TestString:= StrLoadFile_Test3(demofile);
  stop := Clock;
  writeln('test 3 execution time: ',stop-start);
  
  // now lets see how fast stringlist is
  start := Clock;
  for i := 0 to count do
  begin
    TestStrList:= TStringList.create;
    TestStrList.loadfromfile(demofile);
    TestString:= TestStrList.text;
    TestStrList.free; TestStrList:= nil;
  end;
  stop := Clock;
  writeln('test 4 execution time: ',stop-start);


  writeln;
  writeln('Done.');
  writeln('Press <enter> to exit');
  readln;
end.


